# Final Project
# File: flabby_bird.py
# Authors: Dhruv Gaba (dg566) and Tim Braren (tmb233)
# Date: 11/30/2017

import pygame
import time
import math
from random import randint
from constants import *

# Flags encoding the current state of the game
game_over = 0
once = 1

# Player class
class Player(pygame.sprite.Sprite):
    # Creates an instance of of the Player object
    def __init__(self):
        super(Player,self).__init__()
        self.image = pygame.image.load\
        ("/home/pi/pi_arcade/assets/flabbybird_main.png").convert()
        self.rect = self.image.get_rect()
        self.change_x = 4
        self.change_y = 0
        self.level = None
        self.score = 0

    # Updates the current state of the player and checks to see whether or not
    # any collisions have occurred
    def update(self):
        global game_over
        # Update player's velocity based on gravity
        self.update_gravity()
        self.rect.x += self.change_x

        # Find out what the player has collided with. If collision, then set
        # game over flag high
        if not game_over:
            block_hit_list = []
            block_hit_list = pygame.sprite.spritecollide\
            (self, self.level.barrier_list, False)
            for block in block_hit_list:
                if self.change_x > 0:
                    self.rect.right = block.rect.left
                    self.game_over()

        # If the game is not over and hits the top of the screen then limit
        # the y position of the bird to the playing field
        if self.rect.y + self.change_y < Y_START and not game_over:
            self.rect.y = Y_START
            self.change_y = 1
        else:
            self.rect.y += self.change_y

        # Check tos ee whther or not the bird has collided with barriers
        if not game_over:
            block_hit_list = []
            block_hit_list = pygame.sprite.spritecollide\
            (self, self.level.barrier_list, False)
            for block in block_hit_list:
                if self.change_y > 0:
                    self.rect.bottom = block.rect.top
                    self.game_over()
                elif self.change_y < 0:
                    self.rect.top = block.rect.bottom
                    self.game_over()
                self.change_y = 0
                if isinstance(block, Enemy):
                    self.rect.x += block.change_x

    # Update the bird y-velocity according to gravity
    def update_gravity(self):
        global game_over
        if self.change_y == 0:
            self.change_y = 1
        else:
            self.change_y += .35
        if not game_over:
            if self.rect.y >= SCREEN_HEIGHT - self.rect.height \
            and self.change_y >= 0:
                self.change_y = 0
                self.rect.y = SCREEN_HEIGHT - self.rect.height

    # Make the bird jump by fixing its y-velocity to larger negative value
    def jump(self):
        self.rect.y += 2
        barrier_hit_list = pygame.sprite.spritecollide\
        (self, self.level.barrier_list, False)
        self.rect.y -= 2
        self.change_y = -9

    # Move the bird further right on the side-scroller
    def go_right(self):
        self.change_x = 6

    # End the game
    def game_over(self):
        global game_over
        self.change_x = 0
        self.change_y = 1
        game_over = 1

# Barrier Class. Used to construct the walls with holes in them
class Barrier(pygame.sprite.Sprite):
    def __init__(self, width, height):
        super(Barrier,self).__init__()
        self.image = pygame.Surface([width, height])
        self.image.fill(RED)
        self.rect = self.image.get_rect()

# Wall defines two barriers with a hole for the bird to fly through
class Wall(pygame.sprite.Sprite):
    def __init__(self, height1, height2):
        self.bottom = Barrier(40,height1)
        self.top = Barrier(40,SCREEN_HEIGHT-40-height2)

# Enemy class for enemy bird sprites that move up and down
class Enemy(pygame.sprite.Sprite):
    def __init__(self, player):
        super(Enemy,self).__init__()
        self.image = pygame.image.load\
        ("/home/pi/pi_arcade/assets/flabbybird_enemy.png").convert()
        self.rect = self.image.get_rect()
        self.change_x = 0
        self.change_y = randint(2,5)
        self.boundary_bottom = 630
        self.boundary_top = Y_START
        self.player = player

    # Update the position of the current enemy bird sprite. If hits top of the
    # gameplay boundary box then revevrse directions. Similarly, if hits bottom
    # of the gameplay boundary then also reverse directions. Velocity is 
    # always constant but is randomized at initialization.
    def update(self):
        self.rect.y += self.change_y
        hit = pygame.sprite.collide_rect(self, self.player)
        if hit and not game_over:
            self.game_over()
        if self.rect.bottom >= self.boundary_bottom \
        or self.rect.top <= self.boundary_top:
            self.change_y *= -1

    # If the game is over then set the player x velocity to zero and the y 
    # velocity to 1 so that it drops with gravity.
    def game_over(self):
        global game_over
        self.player.change_x = 0
        self.player.change_y = 1
        game_over = 1

# Generic Level class. For now there's only one level, but this structure makes
# it simple for developers to create new levels that can be appended to a list
# of levels to execute in the main loop of the program.
class Level(object):
    barrier_list = None
    background = None
    world_shift = 0

    # Initialize an instance of a level object
    def __init__(self, player):
        self.barrier_list = pygame.sprite.Group()
        self.player = player
        self.level_limit = -1000
        self.difficulty = 0

    # Update the current positions of all of the barriers in the game to account
    # for the level shifting
    def update(self):
        self.barrier_list.update()

    # Draw the current level
    def draw(self, screen):
        screen.fill((153,217,234))

        self.barrier_list.draw(screen)

        # Redraw the banner
        banner = pygame.image.load\
        ("/home/pi/pi_arcade/assets/flabbybird_banner.png").convert()
        screen.blit(banner,(0, 0)) 

        # Draw the current score
        font = pygame.font.Font(None,50)
        scoretext=font.render("Score: "+str(int(self.player.score)), 1,(0,0,0))
        screen.blit(scoretext, (600, 10))

    # Shift the entire world by the same speed as the player to create the
    # effect of side-scrolling
    def shift_world(self, shift_x):
        self.world_shift += shift_x
        for barrier in self.barrier_list:
            if barrier.rect.x >= 90:
                barrier.rect.x += shift_x
                if barrier.rect.x < 90:
                    self.player.score += 0.5

            elif barrier.rect.x >= -40:
                barrier.rect.x += shift_x

# Create a child class of level for a specific randomly generated level that
# gets progressively harder as time goes on.
class Level1(Level):
    def __init__(self, player):
        Level.__init__(self, player)
        self.level_limit = -50000
        self.difficulty = 0
        self.enemies = []

        level = []
        x = 100
        min_slot_width = 100
        slot_width = 250
        x_dist = 100

        # Create a list of arrays that encode the x locations of each of the
        # walls in the game, as well as the heights of the individual barriers
        # that constitute each wall. Each entry can also encode the x-location
        # of an enemy bird.
        while x < abs(self.level_limit):
            dx = randint(170+x_dist,300+x_dist)
            x = x + dx
            # Create a randomized barrier height but ensure that there's always
            # enough clearance for a player to theoretically jump through the
            # gap and make it to the next wall. Use counters to progressively
            # decrease this gap over time so that the game gets harder and 
            # harder.
            height1 = randint(0,560-slot_width)
            height2 = randint(height1+randint(slot_width,560-height1),560)
            level.append([height1, height2, x])
            if slot_width > min_slot_width:
                slot_width -= 3
            if x_dist > 0:
                x_dist -= 5

        # Make 1/3 of the sprites in the game enemy birds, and the remaining
        # 2/3rds as walls.
        for sprite in level:
            draw_wall = (randint(0,2) != 0)
            if draw_wall:
                barrier = Wall(sprite[0], sprite[1])
                barrier.bottom.rect.x = sprite[2]
                barrier.bottom.rect.y = \
                SCREEN_HEIGHT - barrier.bottom.rect.height
                self.barrier_list.add(barrier.bottom)
                barrier.top.rect.x = sprite[2]
                barrier.top.rect.y = Y_START
                self.barrier_list.add(barrier.top)
            else:
                enemy = Enemy(player)
                enemy.rect.right = sprite[2]
                enemy.rect.bottom = \
                randint(Y_START + enemy.rect.height, SCREEN_HEIGHT)
                self.enemies.append(enemy)

# Main function for flabby_bird program
def main():
    global game_over, once
    pygame.init()

    # Initialize the screen width and height and reset the pygame display mode
    size = [SCREEN_WIDTH, SCREEN_HEIGHT]
    screen = pygame.display.set_mode(size)

    # Create a player instance for the current player
    player = Player()

    # Create an instance of a custom level for the player to play
    current_level = Level1(player)
    player.level = current_level

    player.rect.x = 0
    player.rect.y = SCREEN_HEIGHT - player.rect.height

    sprites = pygame.sprite.Group()
    sprites.add(player)

    # Add all of the enemy sprites to the list of sprites in the game to be 
    # updated at each while-loop interation
    for enemy in current_level.enemies:
        sprites.add(enemy)

    done = False
    clock = pygame.time.Clock()

    # Main while-loop of the program
    while not done:
        # Check the posted pygame event flags to see if the player has pressed
        # the jump button
        for e in pygame.event.get():
            if e.type == 23 and not game_over:
                player.jump()
            elif e.type == pygame.KEYDOWN:
                if e.key == pygame.K_UP and not game_over:
                    player.jump()
            if e.type == pygame.QUIT:
                done = True

        # Update all of the player and enemy sprites as well as the walls
        sprites.update()
        current_level.update()

        # Make sure that the player can only go so far on the screen
        if player.rect.right >= 150:
            diff = player.rect.right - 150
            player.rect.right = 150

            # Change the positions of all enemies to account for world-shift
            for enemy in current_level.enemies:
                if enemy.rect.right >= 110:
                    enemy.rect.right -= diff
                    if enemy.rect.right < 110:
                        player.score += 1
                elif enemy.rect.right >= 0:
                    enemy.rect.right -= diff

            sprites = pygame.sprite.Group()
            sprites.add(player)

            # Add each updated enemy
            for enemy in current_level.enemies:
                sprites.add(enemy)

            # Shift the world
            current_level.shift_world(-diff)
            
        # Change the current player position to account for world-shift
        current_position = player.rect.x + current_level.world_shift
        if current_position < current_level.level_limit:
            done = True

        # Redraw the screen and tick the 60fps clock
        current_level.draw(screen)
        sprites.draw(screen)
        clock.tick(60)

        pygame.display.flip()

        # If the game is over and the bird is falling, wait 2.5s and then
        # display the game over screen and play the game over music
        if game_over and once:
            start_time = time.time()
            once = 0

        if not once and time.time() - start_time > 2.25:
            done = 1


    # Play the game over music, display the final score, and write to screen
    stage_music("game_over.wav")
    start_music(0)

    # Display that the game is over and display the score/high scores
    font = pygame.font.Font(None,100)
    game_over = font.render("GAME OVER", 1,(255,255,255))
    score_text = font.render("Score: "+str(int(player.score)), 1,(255,255,255))
    screen.fill((0,0,0))
    screen.blit(game_over, (200, 50))
    screen.blit(score_text, (260, 125))
    pygame.display.flip()
    write_score("flabby_bird", player.score)

    # Wait for the user to press enter before returning to the main screen
    while(1):
        # Get the next event
        e = pygame.event.wait()
        try:
            if e.type == 23:
                break
            elif e.key == pygame.K_RETURN:
                break
        except:
            pass
    return

def flappy_bird_game():
    global game_over, once
    game_over = 0
    once = 1
    main()
    return
