#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Final Project
# File: pi_arcade.py
# Authors: Dhruv Gaba (dg566) and Tim Braren (tmb233)
# Date: 11/30/2017

# Import libraries
import time
import subprocess
import RPi.GPIO as GPIO
import pygame
from pygame.locals import*
from pygame.key import *
import os
import random
import sys
import math
from menu import *
from snake import *
from flappy_bird import flappy_bird_game
from pong import *

# Callbacks for all of the arcade controls
def up(channel):
    pygame.event.post(pygame.event.Event(17))
    
def down(channel):
    pygame.event.post(pygame.event.Event(18))
    
def left(channel):
    pygame.event.post(pygame.event.Event(20))

def right(channel):
    pygame.event.post(pygame.event.Event(19))

def start(channel):
    pygame.event.post(pygame.event.Event(31))

def select(channel):
    pygame.event.post(pygame.event.Event(22))

def a(channel):
    pygame.event.post(pygame.event.Event(23))
    
def b(channel):
    pygame.event.post(pygame.event.Event(24))

def x(channel):
    pygame.event.post(pygame.event.Event(32))

def y(channel):
    pygame.event.post(pygame.event.Event(26))

def l(channel):
    pygame.event.post(pygame.event.Event(27))

def r(channel):
    pygame.event.post(pygame.event.Event(28))
    
def extra1(channel):
    #pygame.event.post(pygame.event.Event(29))
    os.system("sudo poweroff")
 
def extra2(channel):
    #pygame.event.post(pygame.event.Event(30))
    quit_arcade()
    sys.exit(0)
 
screen = None

# Initializes the arcade screen and ignores mouse
def arcade_init():
    # Create the game screen
    global time
    global screen
    screen = pygame.display.set_mode((800, 630))
    pygame.display.set_caption("Pi Arcade")

    # Ignore mouse cursor
    pygame.mouse.set_visible(False)

    # Ignore mouse
    pygame.event.set_blocked(pygame.MOUSEMOTION)
    
    time = pygame.time.Clock()

# Main menu screen of the Pi Arcade
def main_menu():
    global selected
    global menu
    global time
    global screen

    initial_idx = 0
    c_idx = 0
    c_list = ["A","B","C","D","E","F","G","H","I","J","K","L","M",
              "N","O","P","Q","R","S","T","U","V","W","X","Y","Z"]
    initials = ""

    screen.fill((0,0,0))
    pygame.display.update()

    title = pygame.image.load(os.path.join\
        ("/home/pi/pi_arcade/assets","title.png"))
    title = title.convert()
    rect_list = []
    rect_list.append(screen.blit(title,(screen.get_rect().centerx-192,10))) 

    font = pygame.font.Font(None,50)
    prompt = font.render("Enter Your Initials", 1,(255,255,255))
    rect_list.append(screen.blit(prompt, (250, 150)))   

    current_letter = font.render(c_list[c_idx], 1,(255,255,255))
    rect_list.append(screen.blit(current_letter, (325 + 50*(initial_idx), 250)))

    # Update the screen
    pygame.display.update(rect_list)


    # Let the user input the initials they want to use for high scores in the 
    # current game session
    while not (initial_idx == 3):
        # Get the next event
        e = pygame.event.wait()
        if e.type == pygame.KEYUP or (e.type >= 17 and e.type <=32):
            try:
                if e.type == 23:
                    # Input the currently selected character, go to next initial
                    initial_idx += 1
                    initials = initials + c_list[c_idx]

                elif e.type == 17:
                    # Up
                    c_idx = (c_idx + 1) % 26

                elif e.type == 18:
                    # Down
                    c_idx = (c_idx - 1) % 26

                elif e.key == pygame.K_RETURN:
                    # Input the currently selected character, go to next initial
                    initial_idx += 1
                    initials = initials + c_list[c_idx]

                elif e.key == pygame.K_UP:
                    # Up
                    c_idx = (c_idx + 1) % 26

                elif e.key == pygame.K_DOWN:
                    # Down
                    c_idx = (c_idx - 1) % 26
            except:
                pass

        # Redraw the currently-input characters for the initials
        image = pygame.Surface([800, 405])
        image.fill(BLACK)
        screen.blit(image,(0,225)) 

        pygame.display.update()

        title = pygame.image.load(os.path.join\
            ("/home/pi/pi_arcade/assets","title.png"))
        title = title.convert()
        rect_list = []
        rect_list.append(screen.blit(title,(screen.get_rect().centerx-192,10))) 

        font = pygame.font.Font(None,50)
        prompt = font.render("Enter Your Initials", 1,(255,255,255))
        rect_list.append(screen.blit(prompt, (250, 150))) 

        for k in range(0,len(initials)):
            letter = initials[k]
            first_letter = font.render(letter, 1,(255,255,255))
            rect_list.append(screen.blit(first_letter, (325 + 50*k, 250)))

        # Draw input letters on screen
        current_letter = font.render(c_list[c_idx], 1,(255,255,255))
        rect_list.append(screen.blit(current_letter, \
            (325 + 50*(initial_idx), 250)))

        # Update the screen
        pygame.display.update(rect_list)

    set_user_name(initials)
    screen.fill((0,0,0))
    pygame.display.update()
    menu = cMenu(0,50,20,10,'vertical',7,screen,
                [('About', 1, None),
                 ('Emulation Station', 2, None),
                 ('Snake', 3, None),
                 ('Flabby Bird', 4, None),
                 ('Pong', 5, None),
                 ('High Scores', 6, None),
                 ('Shutdown', 7, None)])

    # Center the menu on the screen
    menu.set_center(True, True)
    menu.set_alignment('center', 'center')

    # Initialize state variables to be different so that user event is
    # triggered at start of the while-1 loop so that the initial display doesn't 
    # wait for user input
    state = 0
    prev_state = -1

    # rect_list is a list of pygame.Rect's that will tell pygame where to
    # update the screen - only have to update portions of the screen that have
    # changed since the last screen was drawn
    rect_list = []

    title = pygame.image.load(os.path.join\
        ("/home/pi/pi_arcade/assets","title.png"))
    title = title.convert()
    
    # If any music is playing then stop it
    if music_is_on():
        stop_music()
    
    # Prepare music for menu
    stage_music("main_screen.mp3")
    start_music(1)
    toggle = 1
    toggle2 = 1
    
    # The main while loop
    while 1:
      # Check if the state has changed, if it has, then post a user event to
      # the queue to force the menu to be shown at least once
      if prev_state != state:
         pygame.event.post(pygame.event.Event(EVENT_CHANGE_STATE, key = 0))
         prev_state = state

      # Get the next event
      e = pygame.event.wait()
      # Update the menu, based on which "state" we are in - When using the menu
      # in a more complex program, definitely make the states global variables
      # so that you can refer to them by a name
      if e.type == pygame.KEYDOWN or e.type == EVENT_CHANGE_STATE or \
      (e.type >= 17 and e.type <=32):
        if state == -1: 
            # Waiting state
            screen.fill((0,0,0))
            rect_list.append(screen.get_rect())
            menu = cMenu(0,50,20,10,'vertical',7,screen,
                        [('About', 1, None),
                         ('Emulation Station', 2, None),
                         ('Snake', 3, None),
                         ('Flabby Bird', 4, None),
                         ('Pong', 5, None),
                         ('High Scores', 6, None),
                         ('Shutdown', 7, None)])
            menu.set_center(True, True)
            menu.set_alignment('center', 'center')
            state = 0
            prev_state = -1

        elif state == 0:
            rect_list, state = menu.update(e, state)

        elif state == 1:
            # About menu state
            if toggle:
                screen.fill((0,0,0))
                rect_list.append(screen.get_rect())
                menu = cMenu(0,0,20,10,'vertical',3,screen,
                    [('ECE 5725 - Fall 2017',1,None),\
                     ('Authors: Tim Braren and Dhruv Gaba',1,None),\
                     ('Press Enter to Return to Main Menu',1,None)])
                menu.set_center(True, True)
                menu.set_alignment('center', 'center')
                menu.set_selected_color((255,255,255))
                state = 0
                prev_state = -1
                toggle = 0
            else:
                screen.fill((0,0,0))
                rect_list.append(screen.get_rect())
                menu = cMenu(0,50,20,10,'vertical',7,screen,
                        [('About', 1, None),
                         ('Emulation Station', 2, None),
                         ('Snake', 3, None),
                         ('Flabby Bird', 4, None),
                         ('Pong', 5, None),
                         ('High Scores', 6, None),
                         ('Shutdown', 7, None)])
                menu.set_center(True, True)
                menu.set_alignment('center', 'center')
                state = 0
                prev_state = -1
                toggle = 1 

        elif state == 2:
            # Emulation station state, idles until emulator is exited
            if music_is_on():
                stop_music()

            screen.fill((0,0,0))
            pygame.display.update()
            os.system("sudo -H -u pi emulationstation")
            pygame.event.clear()
            screen.fill((0,0,0))
            rect_list.append(screen.get_rect())
            menu = cMenu(0,50,20,10,'vertical',7,screen,
                        [('About', 1, None),
                         ('Emulation Station', 2, None),
                         ('Snake', 3, None),
                         ('Flabby Bird', 4, None),
                         ('Pong', 5, None),
                         ('High Scores', 6, None),
                         ('Shutdown', 7, None)])
            menu.set_center(True, True)
            menu.set_alignment('center', 'center')
            stage_music("main_screen.mp3")
            start_music(1);
            state = 0
            prev_state = -1

        elif state == 3:
            # Snake game state. Plays new music file and then enters snake.py
            # main function
            stage_music("snake_music.wav")
            start_music(1)
            snake_game()
            stage_music("main_screen.mp3")
            start_music(1);
            screen.fill((0,0,0))
            rect_list.append(screen.get_rect())
            menu = cMenu(0,50,20,10,'vertical',7,screen,
                        [('About', 1, None),
                         ('Emulation Station', 2, None),
                         ('Snake', 3, None),
                         ('Flabby Bird', 4, None),
                         ('Pong', 5, None),
                         ('High Scores', 6, None),
                         ('Shutdown', 7, None)])
            menu.set_center(True, True)
            menu.set_alignment('center', 'center')
            state = 0
            prev_state = -1

        elif state == 4:
            # Flabby bird game state. Plays new music file and then enters
            # flappy_bird.py main function.
            stage_music("flabbybird_music.wav")
            start_music(1)
            flappy_bird_game()
            stage_music("main_screen.mp3")
            start_music(1);
            screen.fill((0,0,0))
            rect_list.append(screen.get_rect())
            menu = cMenu(0,50,20,10,'vertical',7,screen,
                        [('About', 1, None),
                         ('Emulation Station', 2, None),
                         ('Snake', 3, None),
                         ('Flabby Bird', 4, None),
                         ('Pong', 5, None),
                         ('High Scores', 6, None),
                         ('Shutdown', 7, None)])
            menu.set_center(True, True)
            menu.set_alignment('center', 'center')
            state = 0
            prev_state = -1

        elif state == 5:
            # Pong game state. Plays new music file and then enters
            # pong.py main function.
            if music_is_on():
                stop_music()
            stage_music("pong_music.wav")
            start_music(1)
            pong_game()
            screen.fill((0,0,0))
            rect_list.append(screen.get_rect())
            menu = cMenu(0,50,20,10,'vertical',7,screen,
                        [('About', 1, None),
                         ('Emulation Station', 2, None),
                         ('Snake', 3, None),
                         ('Flabby Bird', 4, None),
                         ('Pong', 5, None),
                         ('High Scores', 6, None),
                         ('Shutdown', 7, None)])
            menu.set_center(True, True)
            menu.set_alignment('center', 'center')
            stage_music("main_screen.mp3")
            start_music(1);
            state = 0
            prev_state = -1

        elif state == 6:
            # High score display state. Displays high scores for all of the
            # custom games in order of score. Also shows initials.
            if toggle2:
                screen.fill((0,0,0))
                font = pygame.font.Font(None,100)
                high_score_title = font.render("HIGH SCORES", 1,(255,255,255))

                font = pygame.font.Font(None,50)
                flabby = font.render("Flabby Bird ", 1,(255,255,255))
                snake = font.render("Snake ", 1,(255,255,255))

                screen.blit(high_score_title, (140, 180))
                screen.blit(flabby, (170, 300))
                screen.blit(snake, (470, 300))


                flabby_scores = reversed(load_scores("flabby_bird"))
                snake_scores = reversed(load_scores("snake_game"))
                index = 0
                font = pygame.font.Font(None,35)
                for i, s in enumerate(flabby_scores):
                    index += 1
                    st = font.render(s[0]+"     "+str(s[1]), 1,(255,255,255))
                    screen.blit(st, (220, 350 + 35*i))
                    if index > 5:
                        break
                index = 0
                for i, s in enumerate(snake_scores):
                    index += 1
                    st = font.render(s[0]+"     "+str(s[1]), 1,(255,255,255))
                    screen.blit(st, (470, 350 + 35*i))
                    if index > 5:
                        break
                index = 0
                rect_list.append(screen.get_rect())
                toggle2 = 0

            else:
                screen.fill((0,0,0))
                rect_list.append(screen.get_rect())
                menu = cMenu(0,50,20,10,'vertical',7,screen,
                        [('About', 1, None),
                         ('Emulation Station', 2, None),
                         ('Snake', 3, None),
                         ('Flabby Bird', 4, None),
                         ('Pong', 5, None),
                         ('High Scores', 6, None),
                         ('Shutdown', 7, None)])
                menu.set_center(True, True)
                menu.set_alignment('center', 'center')
                state = 0
                prev_state = -1
                toggle2 = 1             

        elif state == 7:
            # Quits the arcade by running poweroff command with sudo
            # privileges
            os.system("sudo poweroff")
            return

      # Quit if the user presses the exit button
      if e.type == pygame.QUIT:
         quit_arcade()
         return

      rect_list.append(screen.blit(title,(screen.get_rect().centerx-192,10)))  

      # Update the screen
      pygame.display.update(rect_list)
   
def quit_arcade():
    pygame.display.quit()

if __name__ == "__main__": 
    # Set GPIO mode to Broadcom
    GPIO.setmode(GPIO.BCM)

    # # Setup inputs
    GPIO.setup(10, GPIO.IN, GPIO.PUD_UP) #up
    GPIO.setup(22, GPIO.IN, GPIO.PUD_UP) #down
    GPIO.setup(27, GPIO.IN, GPIO.PUD_UP) #left
    GPIO.setup(17, GPIO.IN, GPIO.PUD_UP) #right
    GPIO.setup(16, GPIO.IN, GPIO.PUD_UP) #start
    GPIO.setup(20, GPIO.IN, GPIO.PUD_UP) #select
    GPIO.setup( 9, GPIO.IN, GPIO.PUD_UP) #A
    GPIO.setup( 5, GPIO.IN, GPIO.PUD_UP) #B
    GPIO.setup(13, GPIO.IN, GPIO.PUD_UP) #X
    GPIO.setup(26, GPIO.IN, GPIO.PUD_UP) #Y
    GPIO.setup( 6, GPIO.IN, GPIO.PUD_UP) #L
    GPIO.setup( 4, GPIO.IN, GPIO.PUD_UP) #R
    GPIO.setup(12, GPIO.IN, GPIO.PUD_UP) #extra1
    GPIO.setup(23, GPIO.IN, GPIO.PUD_UP) #extra2

    # # Add callbacks
    GPIO.add_event_detect(10, GPIO.FALLING, callback=up, bouncetime=300)
    GPIO.add_event_detect(22, GPIO.FALLING, callback=down, bouncetime=300)
    GPIO.add_event_detect(27, GPIO.FALLING, callback=left, bouncetime=300)
    GPIO.add_event_detect(17, GPIO.FALLING, callback=right, bouncetime=300)
    GPIO.add_event_detect(16, GPIO.FALLING, callback=start, bouncetime=300)
    GPIO.add_event_detect(20, GPIO.FALLING, callback=select, bouncetime=300)
    GPIO.add_event_detect(9, GPIO.FALLING, callback=a, bouncetime=300)
    GPIO.add_event_detect(5, GPIO.FALLING, callback=b, bouncetime=300)
    GPIO.add_event_detect(13, GPIO.FALLING, callback=x, bouncetime=300)
    GPIO.add_event_detect(26, GPIO.FALLING, callback=y, bouncetime=300)
    GPIO.add_event_detect(6, GPIO.FALLING, callback=l, bouncetime=300)
    GPIO.add_event_detect(4, GPIO.FALLING, callback=r, bouncetime=300)
    GPIO.add_event_detect(12, GPIO.FALLING, callback=extra1, bouncetime=300)
    GPIO.add_event_detect(23, GPIO.FALLING, callback=extra2, bouncetime=300)

    # Initialize pygame mixer
    pygame.mixer.pre_init(44100,-16,2,1024)
    pygame.init()
    arcade_init()
    main_menu()
