# Final Project
# File: pong.py
# Authors: Dhruv Gaba (dg566) and Tim Braren (tmb233)
# Date: 11/30/2017

import RPi.GPIO as GPIO
import random
import pygame
import sys
import time
from pygame.locals import *
from constants import *

B_RADIUS = 10
PADDLE_WIDTH = 12
PADDLE_HEIGHT = 100
HALF_PADDLE_WIDTH = PADDLE_WIDTH / 2
HALF_PADDLE_HEIGHT = PADDLE_HEIGHT / 2
p1_score = 0
p2_score = 0
p1_velocity = 0
p2_velocity = 0
ball_position = [0,0]
ball_velocity = [0,0]

# Spawn a new ball at the center of the field with the input direction. If the
# input direction is 1 then spawn to the right, otherwise spawn to the left
def new_ball(dir):
    global ball_position, ball_velocity

    vertical_velocity = random.randrange(2,5)
    horizontal_velocity = random.randrange(3,6)
    ball_position = [SCREEN_WIDTH/2,SCREEN_HEIGHT/2 + 35]
    
    if dir == 0:
        horizontal_velocity = -horizontal_velocity

    ball_velocity = [horizontal_velocity,-vertical_velocity]

# Draw a new screen based on the game updates
def draw(screen):
    global paddle1_pos, paddle2_pos, ball_position
    global ball_velocity, p1_score, p2_score

    # Fill the screen with black, redraw the paddles, field, banner, etc.
    screen.fill(BLACK)
    pygame.draw.line(screen, WHITE, [SCREEN_WIDTH / 2, 70],[SCREEN_WIDTH / 2, \
        SCREEN_HEIGHT], 1)
    pygame.draw.circle(screen, WHITE, [SCREEN_WIDTH//2, \
        SCREEN_HEIGHT//2 + 35], 100, 1)

    # Update the paddles' vertical positions
    if p1_velocity < 0:
        if paddle1_pos[1] + p1_velocity > HALF_PADDLE_HEIGHT+70:
            paddle1_pos[1] += p1_velocity
        else:
            paddle1_pos[1] = HALF_PADDLE_HEIGHT+70+1

    elif p1_velocity > 0:
        if paddle1_pos[1] + p1_velocity < SCREEN_HEIGHT - HALF_PADDLE_HEIGHT:
            paddle1_pos[1] += p1_velocity
        else:
            paddle1_pos[1] = SCREEN_HEIGHT - HALF_PADDLE_HEIGHT - 1

    if p2_velocity < 0:
        if paddle2_pos[1] + p2_velocity > HALF_PADDLE_HEIGHT + 70:
            paddle2_pos[1] += p2_velocity
        else:
            paddle2_pos[1] = HALF_PADDLE_HEIGHT + 70 + 1

    elif p2_velocity > 0:
        if paddle2_pos[1] + p2_velocity < SCREEN_HEIGHT - HALF_PADDLE_HEIGHT:
            paddle2_pos[1] += p2_velocity
        else:
            paddle2_pos[1] = SCREEN_HEIGHT - HALF_PADDLE_HEIGHT - 1

    # Update the position of the ball
    ball_position[0] = ball_position[0] + int(ball_velocity[0])
    ball_position[1] = ball_position[0] + int(ball_velocity[1])

    # Redraw the paddles and the ball
    pygame.draw.circle(screen, WHITE, ball_position, 20, 0)

    pygame.draw.polygon(screen, RED, [[paddle1_pos[0] - HALF_PADDLE_WIDTH, \
        paddle1_pos[1] - HALF_PADDLE_HEIGHT], \
        [paddle1_pos[0] - HALF_PADDLE_WIDTH, \
        paddle1_pos[1] + HALF_PADDLE_HEIGHT], \
        [paddle1_pos[0] + HALF_PADDLE_WIDTH, \
        paddle1_pos[1] + HALF_PADDLE_HEIGHT], \
        [paddle1_pos[0] + HALF_PADDLE_WIDTH, \
        paddle1_pos[1] - HALF_PADDLE_HEIGHT]], 0)

    pygame.draw.polygon(screen, RED, [[paddle2_pos[0] - HALF_PADDLE_WIDTH, \
        paddle2_pos[1] - HALF_PADDLE_HEIGHT], \
        [paddle2_pos[0] - HALF_PADDLE_WIDTH, \
        paddle2_pos[1] + HALF_PADDLE_HEIGHT], \
        [paddle2_pos[0] + HALF_PADDLE_WIDTH, \
        paddle2_pos[1] + HALF_PADDLE_HEIGHT], \
        [paddle2_pos[0] + HALF_PADDLE_WIDTH, \
        paddle2_pos[1] - HALF_PADDLE_HEIGHT]], 0)

    # Check to see if there's a collision on the top or bottom walls
    if int(ball_position[1]) <= B_RADIUS + 70:
        ball_velocity[1] = - ball_velocity[1]

    if int(ball_position[1]) >= SCREEN_HEIGHT - B_RADIUS - 1:
        ball_velocity[1] = -ball_velocity[1]
    
    # Check to see if ball has hit paddle and update accordingly
    if int(ball_position[0]) <= B_RADIUS + PADDLE_WIDTH and \
    int(ball_position[1]) in range(paddle1_pos[1] - HALF_PADDLE_HEIGHT, \
        paddle1_pos[1] + HALF_PADDLE_HEIGHT,1):
        ball_velocity[0] = -ball_velocity[0]
        ball_velocity[0] = ball_velocity[0]*1.2
        ball_velocity[1] = ball_velocity[1]*1.2
    elif int(ball_position[0]) <= B_RADIUS + PADDLE_WIDTH:
        p2_score += 1
        new_ball(True)
        
    if int(ball_position[0]) >= SCREEN_WIDTH + 1 - B_RADIUS - PADDLE_WIDTH and \
    int(ball_position[1]) in range(paddle2_pos[1] - HALF_PADDLE_HEIGHT, \
        paddle2_pos[1] + HALF_PADDLE_HEIGHT,1):
        ball_velocity[0] = -ball_velocity[0]
        ball_velocity[0] = ball_velocity[0]*1.2
        ball_velocity[1] = ball_velocity[1]*1.2
    elif int(ball_position[0]) >= SCREEN_WIDTH + 1 - B_RADIUS - PADDLE_WIDTH:
        p1_score += 1
        new_ball(False)

    banner = pygame.image.load\
    ("/home/pi/pi_arcade/assets/pong_banner.png").convert()
    screen.blit(banner,(0, 0)) 

    # Update the game scores on the banner
    myfont = pygame.font.SysFont(None, 50)
    label1 = myfont.render("P1 Score: " + str(p1_score), 1, BLACK)
    screen.blit(label1, (60, 16))
    label2 = myfont.render("P2 Score: " + str(p2_score), 1, BLACK)
    screen.blit(label2, (550, 16))  

# Main function for the pong game
def pong_game():
    global paddle1_pos, paddle2_pos, p1_score, p2_score
    global p1_velocity, p2_velocity

    # Initialize fps clock
    fps = pygame.time.Clock()

    # Create a new screen instance for the pong game
    window = pygame.display.set_mode((SCREEN_WIDTH, SCREEN_HEIGHT), 0, 32)  

    # Initialize the paddle positions, scores, and create a new ball
    paddle1_pos = [HALF_PADDLE_WIDTH - 1,SCREEN_HEIGHT/2 + 35]
    paddle2_pos = [SCREEN_WIDTH +1 - HALF_PADDLE_WIDTH,SCREEN_HEIGHT/2+35]

    p1_score = 0
    p2_score = 0

    if random.randrange(0,2) == 0:
        new_ball(True)
    else:
        new_ball(False)

    done = 0

    while not done:
        # On every iteration of main while loop, redraw screen
        draw(window)
        if(GPIO.input(26) == 0):
            p1_velocity = -5
        if(GPIO.input(5) == 0):
            p1_velocity = 5
        if(GPIO.input(6) == 0):
            p2_velocity = -5
        if(GPIO.input(4) == 0):
            p2_velocity = 5
        if(GPIO.input(26) == 1 and GPIO.input(5) == 1):
            p1_velocity = 0 
        if(GPIO.input(6) == 1 and GPIO.input(4) == 1):
            p2_velocity = 0

        pygame.display.update()

        # Evaluate end game condition to see if game is over
        win_val = 3
        if p1_score >= win_val or p2_score >= win_val:
            # If game is over, play soundbyte and display winner text
            stage_music("game_over.wav")
            start_music(0)
            font = pygame.font.Font(None,100)
            game_over = font.render("GAME OVER", 1,(255,255,255))
            if p1_score >= win_val:
                score_text = font.render("P1 Wins!", 1,(255,255,255))
            elif p2_score >= win_val:
                score_text = font.render("P2 Wins!", 1,(255,255,255))

            # Fill the window and blit/display text
            window.fill((0,0,0))
            window.blit(game_over, (200, 50))
            window.blit(score_text, (260, 125))
            pygame.display.flip()

            # Wait for player to press A or enter to go back to main screen
            while(1):
                # Get the next event
                e = pygame.event.wait()
                try:
                    if e.type == 23:
                        break
                    elif e.key == pygame.K_RETURN:
                        break
                except:
                    pass
            return

        # Tick the clock
        fps.tick(60)
