# Final Project
# File: snake.py
# Authors: Dhruv Gaba (dg566) and Tim Braren (tmb233)
# Date: 11/30/2017

# Import libraries
from random import randint
from pygame.locals import*
from pygame.key import *
from constants import *
import pygame
import time
import sys
import os

# Apple Class
class Apple:
	# Define the initial state for an apple
    def __init__(self,x,y):
        self.step = 40
        self.x = x * self.step
        self.y = y * self.step + Y_START
	
	# Draw the apple at the specified position
    def draw(self, surface, image):
        surface.blit(image,(self.x, self.y)) 

# Player class 
class Player:
	# Define the initial state for a player
    def __init__(self, length):
        self.x = [0]
        self.y = [70]
        self.step = 40
        self.dir = 0
        self.length = 2
        self.max_update_count = 2
        self.update_count = 0
        self.length = length

        # Fill the snake array with dc values
        for i in range(0,500):
            self.x.append(-100)
            self.y.append(-100)
 
        # Set the initial snake positon to be at the top left of the screen
        self.x[0] = 1*self.step
        self.x[1] = 2*self.step
 
	# Update the snake's position on every time step
    def update(self):
        self.update_count += 1
        if self.update_count > self.max_update_count:
            # Update all of the existing entries in the snake array
            for i in range(self.length-1, 0, -1):
                self.x[i] = self.x[i-1]
                self.y[i] = self.y[i-1]

            # Update the location of the head of the snake according
            # to the current direction that the snake is facing
            if self.dir == 0:
                if self.x[0] == SCREEN_WIDTH - self.step:
                    self.x[0] = X_START
                else:
                    self.x[0] = self.x[0] + self.step

            if self.dir == 1:
                if self.x[0] == X_START:
                    self.x[0] = SCREEN_WIDTH - self.step
                else:
                    self.x[0] = self.x[0] - self.step

            if self.dir == 2:
                if self.y[0] == Y_START:
                    self.y[0] = SCREEN_HEIGHT - self.step
                else:
                    self.y[0] = self.y[0] - self.step

            if self.dir == 3:
                if self.y[0] == SCREEN_HEIGHT - self.step:
                    self.y[0] = Y_START
                else:
                    self.y[0] = self.y[0] + self.step

            # Reset the update count
            self.update_count = 0

    def move_right(self):
        self.dir = 0
 
    def move_left(self):
        self.dir = 1
 
    def move_up(self):
        self.dir = 2
 
    def move_down(self):
        self.dir = 3 
 
    def draw(self, surface, image):
        for i in range(0,self.length):
            surface.blit(image,(self.x[i],self.y[i])) 
 
 # Game Class
class Game:
    # Define boolean for collision
    def collision(self,x1,y1,x2,y2,bsize,player):
        if(x1 == x2 and y1 ==y2):
            print(x1)
            print(y1)
            print(x2)
            print(y2)
            return (x1 == x2 and y1 == y2)

    # Define boolean for out of bounds condition
    def out_of_bounds(self, x, y, width, height):
        return (x > SCREEN_WIDTH - 40 or x < X_START or y > SCREEN_HEIGHT - 40 or y < Y_START)

# App Class
class App:
    # Define the initial state for the application
    def __init__(self):
        self.done = False
        self.screen = None
        self.snake = None
        self.apple_image = None
        self.snake_banner = None
        self.game = Game()
        self.player = Player(2) 
        self.apple = Apple(2,2)
        self.score = 0
 
    def draw_gui(self):
        pygame.init()
        self.screen = pygame.display.set_mode((SCREEN_WIDTH,SCREEN_HEIGHT), pygame.HWSURFACE)
        self.done = False
        self.snake_banner = pygame.image.load("/home/pi/pi_arcade/assets/snake_banner.png").convert()
        self.snake = pygame.image.load("/home/pi/pi_arcade/assets/snake.png").convert()
        self.apple_image = pygame.image.load("/home/pi/pi_arcade/assets/apple.png").convert()
 
    def update_game(self):
        self.player.update()
 
        # Check to see if the snake has gotten the food
        for i in range(0,self.player.length):
            if self.game.collision(self.apple.x,self.apple.y,self.player.x[i],self.player.y[i],self.player.step,self.player):
                self.apple.x = randint(0,19) * self.player.step
                self.apple.y = randint(0,13) * self.player.step + Y_START
                self.player.length = self.player.length + 1
                self.score += 1
 
        # Check to see if the snake collides with itself
        for i in range(1,self.player.length-1):
            if self.game.collision(self.player.x[0],self.player.y[0],self.player.x[i],self.player.y[i],self.player.step,self.player) and self.player.length>4:
                self.done = True

    def update_score(self, score):
        font=pygame.font.Font(None,50)
        scoretext=font.render("Score: "+str(score), 1,(255,255,255))
        self.screen.blit(scoretext, (600, 10))     
 
    def update_gui(self):
        self.screen.fill((0,128,0))
        self.player.draw(self.screen, self.snake)
        self.apple.draw(self.screen, self.apple_image)
        self.screen.blit(self.snake_banner,(0, 0)) 
        self.update_score(self.score)
        pygame.display.flip()
 
    def start_game(self):
        self.draw_gui()
        while not self.done:
            self.update_game()
            self.update_gui()
            for e in pygame.event.get():
            #e = pygame.event.poll()
                if (e.type >= 17 and e.type <= 20):
                    if e.type == 17 and self.player.dir != 3:
                        self.player.move_up() 
                    elif e.type == 18 and self.player.dir != 2:
                        self.player.move_down() 
                    elif e.type == 19 and self.player.dir != 1:
                        self.player.move_right() 
                    elif e.type == 20 and self.player.dir != 0:
                        self.player.move_left()
                else:
                    try:
                        if e.key == pygame.K_UP and self.player.dir != 3:
                            self.player.move_up() 
                        elif e.key == pygame.K_DOWN and self.player.dir != 2:
                            self.player.move_down() 
                        elif e.key == pygame.K_RIGHT and self.player.dir != 1:
                            self.player.move_right() 
                        elif e.key == pygame.K_LEFT and self.player.dir != 0:
                            self.player.move_left()
                    except:
                        pass
            #self.update_game()
            #self.update_gui()
            time.sleep(0.05);

        stage_music("game_over.wav")
        start_music(0)
        # Display that the game is over and display the score/high scores
        font = pygame.font.Font(None,100)
        game_over = font.render("GAME OVER", 1,(255,255,255))
        score_text = font.render("Score: "+str(self.score), 1,(255,255,255))
        self.screen.fill((0,0,0))
        self.screen.blit(game_over, (200, 50))
        self.screen.blit(score_text, (260, 125))
        pygame.display.flip()
        write_score("snake_game", self.score)

        while(1):
            # Get the next event
            e = pygame.event.wait()
            try:
                if e.type == 23:
                    break
                elif e.key == pygame.K_RETURN:
                    break
            except:
                pass
        return

def snake_game():
    game_instance = App()
    game_instance.start_game()
    return
